@@ -1,6 +1,8 @@
 from collections import OrderedDict
 from dataclasses import dataclass
 from typing import Dict, List, Optional
+from pathlib import Path
+
 
 import h5py
 import numpy as np
@@ -14,6 +16,8 @@ from ronswanson.utils.color import Colors
 
 from .utils.logging import setup_logger
 
+from joblib import Parallel, delayed
+
 log = setup_logger(__name__)
 
 
@@ -75,32 +79,79 @@ class Database:
 
         self._meta_data: Optional[Dict[str, np.ndarray]] = meta_data
 
+        if np.any(~np.isfinite(self._values)):
+
+            log.error("The table values contain non-finite values")
+            log.info(
+                "you can replace these by calling the replace_nan_inf_with() member"
+            )
+
     @property
     def n_entries(self) -> int:
+        """
+        the number of entries in the database
+
+        :returns:
+
+        """
+
         return self._n_entries
 
     @property
     def n_parameters(self) -> int:
+        """
+        the number of parameters in the database
+
+        :returns:
+
+        """
+
         return self._n_parameters
 
+    @property
+    def grid_points(self) -> np.ndarray:
+        return self._grid_points
+
+    @property
+    def values(self) -> np.ndarray:
+        return self._values
+
     @property
     def parameter_ranges(self) -> Dict[str, np.ndarray]:
         return self._parameter_ranges
 
     @property
     def parameter_names(self) -> List[str]:
+        """
+        the names of the parameters
+        """
         return self._parameter_names
 
     @property
     def run_time(self) -> np.ndarray:
+        """
+        an array of runtimes for all points in the grid
+
+        :returns:
+
+        """
+
         return self._run_time
 
     @property
     def energy_grid(self) -> np.ndarray:
+        """
+        The corresponding energy grid of the simulation
+
+        :returns:
+
+        """
+
         return self._energy_grid
 
     @property
     def meta_data(self) -> Optional[Dict[str, np.ndarray]]:
+
         return self._meta_data
 
     def at(self, i: int) -> ValueContainer:
@@ -216,6 +267,20 @@ class Database:
             meta_data=meta_data,
         )
 
+    def replace_nan_inf_with(self, value: float = 0.0) -> None:
+
+        """
+        Replace NaN and inf values with a float
+
+        :param value:
+        :type value: float
+        :returns:
+
+        """
+        idx = np.isinf(self._values) | np.isnan(self._values)
+
+        self._values[idx] = value
+
     def to_3ml(
         self,
         name: str,
@@ -465,6 +530,74 @@ class Database:
             raise RuntimeError(msg)
 
 
+def update_database(
+    database_filename: str,
+    sim_number_to_replace: np.ndarray,
+    sim_locations: str,
+    create_backup: bool = True,
+) -> None:
+
+    """
+    will replace the simulations (value arrays) of the indices supplied
+    from the folder indicated. This is useful when certain runs of a
+    simulation failed and one wants to replace those.
+
+    Note: this requires that the sim ids of the replacement runs match those
+    in the database!
+
+    :param database_filename:
+    :type database_filename: str
+    :param sim_number_to_replace:
+    :type sim_number_to_replace: np.ndarray
+    :param sim_locations:
+    :type sim_locations: str
+    :returns:
+
+    """
+    import shutil
+
+    database_path: Path = Path(database_filename).absolute()
+
+    if create_backup:
+
+        backup_file_name = (
+            database_path.parents / f"{database_path.stem}_bkup.h5"
+        )
+
+        log.info(f"creating a backup as {backup_file_name}")
+
+        shutil.copy(database_path, backup_file_name)
+
+    with open(database_path.as_posix(), "r+") as f:
+
+        n_output = 0
+        for key in list(f.keys()):
+
+            if "output_" in key:
+                n_output += 1
+
+        n_meta = 0
+        for key in list(f.attrs.keys()):
+            if "meta_" in key:
+                n_meta += 1
+
+        for idx in tqdm(sim_number_to_replace, desc="replaceing values"):
+
+            with open(
+                Path(sim_locations).absolute() / f"sim_store_{idx}.h5", "r"
+            ) as r:
+
+                for i in range(n_output):
+
+                    f[f"values/output_{i}"][idx, :] = r[f"output_{i}"][()]
+
+                for i in range(n_meta):
+
+                    f[f"meta/meta_{i}"][idx, :] = r.attrs[f"meta_{i}"]
+
+                f["runtime"][idx] = r.attrs["runtime"]
+
+
 def merge_databases(
     *file_names: List[str], new_name: str = "merged_db.h5"
 ) -> None:
